/*
 * Souffle - A Datalog Compiler
 * Copyright (c) 2018, The Souffle Developers. All rights reserved
 * Licensed under the Universal Permissive License v 1.0 as shown at:
 * - https://opensource.org/licenses/UPL
 * - <souffle root>/licenses/SOUFFLE-UPL.txt
 */

/************************************************************************
 *
 * @file ExpandFilter.cpp
 *
 ***********************************************************************/

#include "ram/transform/ExpandFilter.h"
#include "ram/Node.h"
#include "ram/Operation.h"
#include "ram/Program.h"
#include "ram/Statement.h"
#include "ram/Utils.h"
#include "ram/Visitor.h"
#include "souffle/utility/MiscUtil.h"
#include <functional>
#include <memory>
#include <utility>
#include <vector>

namespace souffle {
class Condition;

namespace ram::transform {

bool ExpandFilterTransformer::expandFilters(Program& program) {
    bool changed = false;
    visitDepthFirst(program, [&](const Query& query) {
        std::function<Own<Node>(Own<Node>)> filterRewriter = [&](Own<Node> node) -> Own<Node> {
            if (const Filter* filter = dynamic_cast<Filter*>(node.get())) {
                const Condition* condition = &filter->getCondition();
                VecOwn<Condition> conditionList = toConjunctionList(condition);
                if (conditionList.size() > 1) {
                    changed = true;
                    VecOwn<Filter> filters;
                    for (auto& cond : conditionList) {
                        if (filters.empty()) {
                            filters.emplace_back(mk<Filter>(
                                    souffle::clone(cond), souffle::clone(&filter->getOperation())));
                        } else {
                            filters.emplace_back(mk<Filter>(souffle::clone(cond), std::move(filters.back())));
                        }
                    }
                    node = std::move(filters.back());
                }
            }
            node->apply(makeLambdaRamMapper(filterRewriter));
            return node;
        };
        const_cast<Query*>(&query)->apply(makeLambdaRamMapper(filterRewriter));
    });
    return changed;
}

}  // namespace ram::transform
}  // end of namespace souffle
